<?php

use QUI\Locations\Handler\Locations as LocationsHandler;

/**
 * Create a media folder for location images
 *
 * @param int $locationId - location ID
 * @return false|string - false on error or media folder URL on success
 */
QUI::$Ajax->registerFunction(
    'package_quiqqer_locations_ajax_attributes_MediaFolder_create',
    function ($locationId) {
        try {
            $Location             = LocationsHandler::getLocation((int)$locationId);
            $ImageFolderAttribute = $Location->getLocationAttribute('imagesMediaFolder');

            if (!empty($ImageFolderAttribute->getValue())) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.mediafolder.already.created'
                ]);
            }

            $MainFolder = LocationsHandler::getMainMediaFolder();

            if (!$MainFolder) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.mediafolder.no.main.folder.assigned'
                ]);
            }

            $folderName = QUI::getLocale()->get(
                'quiqqer/locations',
                'attribute.imagesMediaFolder.folder.name.prefix'
            );

            $folderName .= '_'.$Location->getId();

            $NewFolder = $MainFolder->createFolder($folderName);
            $NewFolder->activate();

            $ImageFolderAttribute->setValue($NewFolder->getUrl());
            $Location->save();
        } catch (QUI\Locations\Exception $Exception) {
            QUI::getMessagesHandler()->addError(
                QUI::getLocale()->get(
                    'quiqqer/locations',
                    'message.ajax.attributes.MediaFolder.create.error',
                    [
                        'locationId' => $locationId,
                        'error'      => $Exception->getMessage()
                    ]
                )
            );

            return false;
        } catch (QUI\Permissions\Exception $Exception) {
            QUI::getMessagesHandler()->addError(
                QUI::getLocale()->get(
                    'quiqqer/locations',
                    'message.ajax.attributes.MediaFolder.create.error',
                    [
                        'locationId' => $locationId,
                        'error'      => $Exception->getMessage()
                    ]
                )
            );

            return false;
        } catch (\Exception $Exception) {
            QUI\System\Log::addError(
                'AJAX :: package_quiqqer_locations_ajax_attributes_MediaFolder_create -> '.$Exception->getMessage()
            );

            QUI::getMessagesHandler()->addError(
                QUI::getLocale()->get(
                    'quiqqer/locations',
                    'message.locations.general.error'
                )
            );

            return false;
        }

        QUI::getMessagesHandler()->addSuccess(
            QUI::getLocale()->get(
                'quiqqer/locations',
                'message.ajax.attributes.MediaFolder.create.success',
                [
                    'locationId'   => $locationId,
                    'locationName' => $Location->getLocationAttribute('name')
                ]
            )
        );

        return $NewFolder->getUrl();
    },
    ['locationId'],
    'Permission::checkAdminUser'
);
