<?php

use QUI\Locations\Utils;
use QUI\Locations\Handler\Locations as LocationsHandler;
use QUI\Locations\Exception as LocationException;

/**
 * Create a new location
 *
 * @param string $locationName - new location name
 * @return int|false - new location ID; false on error
 */
QUI::$Ajax->registerFunction(
    'package_quiqqer_locations_ajax_locations_create',
    function ($locationName) {
        $locationName = Utils::clearJSONString($locationName);

        try {
            $newLocationId = LocationsHandler::createLocation($locationName);
        } catch (LocationException $LocationException) {
            QUI::getMessagesHandler()->addError(
                QUI::getLocale()->get(
                    'quiqqer/locations',
                    'message.ajax.createLocation.error',
                    array(
                        'error' => $LocationException->getMessage()
                    )
                )
            );

            return false;
        } catch (\Exception $Exception) {
            QUI\System\Log::addError(
                'AJAX :: package_quiqqer_locations_ajax_locations_create -> ' . $Exception->getMessage()
            );

            return false;
        }

        QUI::getMessagesHandler()->addSuccess(
            QUI::getLocale()->get(
                'quiqqer/locations',
                'message.ajax.createLocation.success'
            )
        );

        return $newLocationId;
    },
    array('locationName'),
    'Permission::checkAdminUser'
);
