<?php

use QUI\Locations\Handler\Locations as LocationsHandler;
use QUI\Lock\Locker;
use QUI\Watcher;

/**
 * Unlocks a location panel if the user has the necessary permission(s)
 *
 * @param string $lockKey - Location panel lock key
 * @return bool - success
 *
 * @throws \QUI\Permissions\Exception
 */
QUI::$Ajax->registerFunction(
    'package_quiqqer_locations_ajax_locations_unlock',
    function ($locationId, $lockKey) {
        $Package = QUI::getPackage('quiqqer/locations');

        try {
            if (Locker::isLocked($Package, $lockKey)) {
                Locker::unlockWithPermissions($Package, $lockKey, LocationsHandler::PERM_FORCE_EDIT_LOCATION);
                Watcher::addString(
                    QUI::getLocale()->get(
                        'quiqqer/locations',
                        'watcher.location.force.edit',
                        array(
                            'locationId' => $locationId,
                            'lockKey'    => $lockKey
                        )
                    ),
                    'package_quiqqer_locations_ajax_locations_lock'
                );
            } else {
                Locker::unlock($Package, $lockKey);
            }
        } catch (\QUI\Permissions\Exception $Exception) {
            return true;
        } catch (\Exception $Exception) {
            QUI\System\Log::addError(
                'AJAX :: package_quiqqer_locations_ajax_locations_lock -> ' . $Exception->getMessage()
            );

            QUI::getMessagesHandler()->addError(
                QUI::getLocale()->get(
                    'quiqqer/locations',
                    'message.locations.general.error'
                )
            );

            return false;
        }

        return true;
    },
    array('locationId', 'lockKey'),
    'Permission::checkAdminUser'
);
