<?php

use QUI\Locations\Handler\Locations as LocationsHandler;
use QUI\Locations\Utils;

/**
 * Set attribute values to a single location and save it to the database
 *
 * @param int $locationId - location ID
 * @param array $attributes - attributes with values
 * @return bool - success
 */
QUI::$Ajax->registerFunction(
    'package_quiqqer_locations_ajax_locations_update',
    function ($locationId, $attributes) {
        $attributes = json_decode($attributes, true);

        try {
            $Location = LocationsHandler::getLocation((int)$locationId);
            $Location->setLocationAttributes($attributes);
            $Location->save();
        } catch (QUI\Locations\Exception $Exception) {
            QUI::getMessagesHandler()->addError(
                QUI::getLocale()->get(
                    'quiqqer/locations',
                    'message.ajax.locations.update.error',
                    array(
                        'locationId' => $locationId,
                        'error'      => $Exception->getMessage()
                    )
                )
            );

            return false;
        } catch (QUI\Permissions\Exception $Exception) {
            QUI::getMessagesHandler()->addError(
                QUI::getLocale()->get(
                    'quiqqer/locations',
                    'message.ajax.locations.update.error',
                    array(
                        'locationId' => $locationId,
                        'error'      => $Exception->getMessage()
                    )
                )
            );

            return false;
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);

            QUI::getMessagesHandler()->addError(
                QUI::getLocale()->get(
                    'quiqqer/locations',
                    'message.locations.general.error'
                )
            );

            return false;
        }

        QUI::getMessagesHandler()->addSuccess(
            QUI::getLocale()->get(
                'quiqqer/locations',
                'message.ajax.locations.update.success',
                array(
                    'locationId'   => $locationId,
                    'locationName' => $Location->getLocationAttribute('name')
                )
            )
        );

        return true;
    },
    array('locationId', 'attributes'),
    'Permission::checkAdminUser'
);
