<?php

use QUI\Locations\Handler\LocationAPIs;
use QUI\Utils\Security\Orthos;

/**
 * Update execution time of location API
 *
 * @return int $id - API ID
 */
QUI::$Ajax->registerFunction(
    'package_quiqqer_locations_ajax_locationsapis_updateExecTime',
    function ($id, $execTime) {
        $execTime = Orthos::clearArray(
            json_decode($execTime, true)
        );

        try {
            LocationAPIs::updateExecTime((int)$id, $execTime);
        } catch (QUI\Locations\Exception $Exception) {
            QUI::getMessagesHandler()->addError(
                QUI::getLocale()->get(
                    'quiqqer/locations',
                    'message.ajax.locationsapis.updateexectime.error',
                    array(
                        'locationApiId' => $id,
                        'error'         => $Exception->getMessage()
                    )
                )
            );

            return false;
        } catch (\Exception $Exception) {
            QUI\System\Log::addError(
                'AJAX :: package_quiqqer_locations_ajax_locationsapis_updateExecTime -> '
                . $Exception->getMessage()
            );

            QUI::getMessagesHandler()->addError(
                QUI::getLocale()->get(
                    'quiqqer/locations',
                    'message.locations.general.error'
                )
            );

            return false;
        }

        QUI::getMessagesHandler()->addSuccess(
            QUI::getLocale()->get(
                'quiqqer/locations',
                'message.ajax.locationsapis.updateexectime.success',
                array(
                    'locationApiId' => $id
                )
            )
        );

        return true;
    },
    array('id', 'execTime'),
    'Permission::checkAdminUser'
);
