/**
 * Locations handler
 *
 * @module package/quiqqer/locations/bin/classes/Locations
 * @author www.pcsg.de (Patrick Müller)
 */
define('package/quiqqer/locations/bin/classes/Locations', [

    'qui/classes/DOM',
    'Ajax'

], function (QUIDOM, QUIAjax) {
    "use strict";

    var pkg = 'quiqqer/locations';

    return new Class({

        Extends: QUIDOM,
        Type   : 'package/quiqqer/locations/bin/classes/Locations',

        /**
         * Search locations
         *
         * @returns {Promise}
         */
        search: function (SearchParams) {
            return new Promise(function (resolve, reject) {
                QUIAjax.get('package_quiqqer_locations_ajax_locations_getList', resolve, {
                    'package'   : pkg,
                    onError     : reject,
                    searchParams: JSON.encode(SearchParams)
                });
            });
        },

        /**
         * Create a new location
         *
         * @param {string} name - location name
         * @returns {Promise}
         */
        createLocation: function (name) {
            return new Promise(function (resolve, reject) {
                QUIAjax.post('package_quiqqer_locations_ajax_locations_create', resolve, {
                    'package'   : pkg,
                    onError     : reject,
                    locationName: name
                });
            });
        },

        /**
         * Get single location
         *
         * @param {number} locationId
         * @returns {Promise}
         */
        getLocation: function (locationId) {
            return new Promise(function (resolve, reject) {
                QUIAjax.post('package_quiqqer_locations_ajax_locations_get', resolve, {
                    'package' : pkg,
                    onError   : reject,
                    locationId: locationId
                });
            });
        },

        /**
         * Activate a Location
         *
         * @param {number} locationId
         * @returns {Promise}
         */
        activateLocation: function (locationId) {
            return new Promise(function (resolve, reject) {
                QUIAjax.post('package_quiqqer_locations_ajax_locations_activate', resolve, {
                    'package' : pkg,
                    onError   : reject,
                    locationId: locationId
                });
            });
        },

        /**
         * Deactivate a Location
         *
         * @param {number} locationId
         * @returns {Promise}
         */
        deactivateLocation: function (locationId) {
            return new Promise(function (resolve, reject) {
                QUIAjax.post('package_quiqqer_locations_ajax_locations_deactivate', resolve, {
                    'package' : pkg,
                    onError   : reject,
                    locationId: locationId
                });
            });
        },

        /**
         * Delete one or more locations
         *
         * @param {array} locationIds
         * @returns {Promise}
         */
        deleteLocations: function (locationIds) {
            return new Promise(function (resolve, reject) {
                QUIAjax.post('package_quiqqer_locations_ajax_locations_delete', resolve, {
                    'package'  : pkg,
                    onError    : reject,
                    locationIds: JSON.encode(locationIds)
                });
            });
        },

        /**
         * Get all columns that are shown in the location overview list
         *
         * @returns {Promise}
         */
        getListColumns: function () {
            return new Promise(function (resolve, reject) {
                QUIAjax.get('package_quiqqer_locations_ajax_locations_getListColumns', resolve, {
                    'package': pkg,
                    onError  : reject
                });
            });
        },

        /**
         * Fetch all available attributes
         *
         * @return {Promise}
         */
        getAttributeList: function () {
            return new Promise(function (resolve, reject) {
                QUIAjax.get('package_quiqqer_locations_ajax_attributes_getList', resolve, {
                    'package': 'quiqqer/locations',
                    onError  : reject
                });
            });
        }
    });
});
