/**
 * Control for setting location coordinates
 *
 * @module package/quiqqer/locations/bin/controls/attributes/Coordinates/Coordinates
 * @author www.pcsg.de (Patrick Müller)
 */
define('package/quiqqer/locations/bin/controls/attributes/Coordinates/Coordinates', [

    'qui/controls/Control',
    'qui/controls/windows/Popup',
    'qui/controls/windows/Confirm',
    'qui/controls/buttons/Button',
    'qui/controls/loader/Loader',

    'Locale',
    'Mustache',

    'package/quiqqer/geoutils/bin/Geo',
    'package/quiqqer/geoutils/bin/classes/GeoPoint',
    'package/quiqqer/geoutils/bin/classes/GeoLocation',

    'text!package/quiqqer/locations/bin/controls/attributes/Coordinates/Coordinates.html',
    'text!package/quiqqer/locations/bin/controls/attributes/Coordinates/Address.html',
    'css!package/quiqqer/locations/bin/controls/attributes/Coordinates/Coordinates.css'

], function (QUIControl, QUIPopup, QUIConfirm, QUIButton, QUILoader, QUILocale,
             Mustache, Geo, GeoPoint, GeoLocation, template, addressTemplate) {
    "use strict";

    var lg = 'quiqqer/locations';

    return new Class({

        Extends: QUIControl,
        Type   : 'package/quiqqer/locations/bin/controls/attributes/Coordinates/Coordinates',

        Binds: [
            '$onInject',
            '$onImport',
            '$build',
            '$getInfoElm',
            '$setValue',
            '$onMapLoaded',
            '$onMapClick',
            '$determineCoordinatesByAddress',
            '$showCoordinates',
            '$showAddressMismatchWarning',
            '$hideAddressMismatchWarning'
        ],

        initialize: function (options) {
            this.parent(options);

            this.addEvents({
                onImport: this.$onImport
            });

            this.Loader = new QUILoader();

            this.$Input       = null;
            this.$Map         = null;
            this.$Location    = null;
            this.$LatInput    = null;
            this.$LngInput    = null;
            this.$MapLocation = null;
        },

        /**
         * create the domnode element
         *
         * @return {HTMLDivElement}
         */
        create: function () {
            this.$Elm = new Element('div', {
                'class': 'quiqqer-locations-attributes-coordinates',
                html   : '<div class="quiqqer-locations-attributes-coordinates-map"></div>' +
                    '<div class="quiqqer-locations-attributes-coordinates-content"></div>'
            });

            this.Loader.inject(this.$Elm);

            return this.$Elm;
        },

        /**
         * Execute when the user clicks on a point of the map
         *
         * @param {package/quiqqer/geoutils/bin/classes/GeoPoint} Coordinates
         */
        $onMapClick: function (Coordinates) {
            var self = this;
            var PreviousCoordinates, Location;

            var Popup = new QUIPopup({
                maxWidth          : 650,
                maxHeight         : 550,
                content           : false,
                icon              : 'fa fa-thumb-tack',
                title             : QUILocale.get(lg, 'controls.attributes.coordinates.popup.title'),
                'class'           : 'quiqqer-locations-attributes-coordinates-popup',
                backgroundClosable: false,

                // buttons
                buttons         : true,
                closeButton     : false,
                titleCloseButton: false,

                events: {
                    onOpen: function () {
                        Popup.getContent().set(
                            'html',
                            '<h1>' +
                            QUILocale.get(lg, 'controls.attributes.coordinates.popup.content.title') +
                            '</h1>' +
                            '<p>' +
                            QUILocale.get(lg, 'controls.attributes.coordinates.popup.content.text') +
                            '</p>' +
                            '<p class="coordinates">' +
                            '<span class="lat">' +
                            QUILocale.get(lg, 'controls.attributes.coordinates.popup.content.lat', {
                                lat: Coordinates.getLatitude()
                            }) +
                            '</span>' +
                            '<span class="lng">' +
                            QUILocale.get(lg, 'controls.attributes.coordinates.popup.content.lng', {
                                lng: Coordinates.getLongitude()
                            }) +
                            '</span>' +
                            '</p>' +
                            '<p class="address">' +
                            QUILocale.get(lg, 'controls.attributes.coordinates.popup.content.address.info') +
                            '</p>'
                        );

                        var countryName     = '-',
                            locationCountry = Location.getAttribute('country');

                        if (locationCountry) {
                            Location.setAttribute('country', locationCountry.toUpperCase());

                            countryName = QUILocale.get(
                                'quiqqer/countries', 'country.' +
                                Location.getAttribute('country')
                            );
                        }

                        var AddressView = {
                            city       : '-',
                            zipCode    : '-',
                            street     : '-',
                            houseNumber: '-'
                        };

                        var LocationAddress = Location.getAddress();

                        for (var property in LocationAddress) {
                            if (!LocationAddress.hasOwnProperty(property)) {
                                continue;
                            }

                            if (LocationAddress[property]) {
                                AddressView[property] = LocationAddress[property];
                            }
                        }

                        new Element('div', {
                            html: Mustache.render(addressTemplate, Object.merge(AddressView, {
                                country         : countryName,
                                countryLabel    : QUILocale.get(lg, 'attribute.country.title'),
                                cityLabel       : QUILocale.get(lg, 'attribute.city.title'),
                                zipCodeLabel    : QUILocale.get(lg, 'attribute.zipCode.title'),
                                streetLabel     : QUILocale.get(lg, 'attribute.street.title'),
                                houseNumberLabel: QUILocale.get(lg, 'attribute.houseNumber.title'),
                                tableHeader     : QUILocale.get(
                                    lg,
                                    'controls.attributes.coordinates.popup.table.header'
                                )
                            }))
                        }).inject(
                            Popup.getContent().getElement(
                                'p.address'
                            )
                        );

                        Popup.addButton(new QUIButton({
                            textimage: 'fa fa-close',
                            text     : QUILocale.get(
                                lg,
                                'controls.attributes.coordinates.popup.content.btn.decline'
                            ),
                            events   : {
                                onClick: function () {
                                    if (PreviousCoordinates) {
                                        self.$Map.setCurrentMarker(PreviousCoordinates);
                                    }

                                    Popup.close();
                                }
                            }
                        }));

                        Popup.addButton(new QUIButton({
                            textimage: 'fa fa-check',
                            text     : QUILocale.get(
                                lg,
                                'controls.attributes.coordinates.popup.content.btn.confirm'
                            ),
                            events   : {
                                onClick: function () {
                                    self.$LatInput.value = Coordinates.getLatitude();
                                    self.$LngInput.value = Coordinates.getLongitude();
                                    self.$MapLocation    = Location;
                                    //self.$Map.setCurrentMarker(Coordinates);
                                    self.$Map.showCoordinates(Coordinates);
                                    self.$setValue();

                                    if (!self.$addressesEqual(self.$Location.getAddress(), Location.getAddress())) {
                                        self.$showAddressMismatchWarning(Location.getAddress());
                                    } else {
                                        self.$hideAdressMismatchWarning();
                                    }

                                    Popup.close();
                                }
                            }
                        }));
                    }
                }
            });

            this.Loader.show();

            Promise.all([
                Geo.getLocationByCoordinates(Coordinates),
                this.$Map.getCurrentMarkerPosition(),
                this.$Map.setCurrentMarker(Coordinates)
            ]).then(function (result) {
                Location            = result[0];
                PreviousCoordinates = result[1];

                Popup.open();
                self.Loader.hide();
            });
        },

        /**
         * event: onImport
         */
        $onImport: function () {
            var self = this;

            this.$Input      = this.getElm();
            this.$Input.type = 'hidden';

            var Elm = self.create();
            Elm.inject(self.$Input, 'after');

            if (this.$Input.getParent('.field-container')) {
                var Container = new Element('div', {
                    'class': 'field-container-field'
                });

                Container.wraps(Elm);
            }

            Geo.getMap({}, true).then(function (Map) {
                Map.addEvents({
                    onMapClick: self.$onMapClick,
                    onLoaded  : self.$onMapLoaded
                });

                self.$Map = Map.inject(
                    self.$Elm.getElement(
                        '.quiqqer-locations-attributes-coordinates-map'
                    )
                );

                if (self.$Map.isInitialized()) {
                    self.$onMapLoaded();
                }
            }, function () {
                self.$Elm.getElement(
                    '.quiqqer-locations-attributes-coordinates-map'
                ).setStyle('display', 'none');
                self.$onMapLoaded();
            });
        },

        /**
         * Execute after map has finished loading
         */
        $onMapLoaded: function () {
            var self = this;

            var waitForLocation = setInterval(function () {
                if (!self.getAttribute('Location')) {
                    return;
                }

                clearInterval(waitForLocation);

                var Location = self.getAttribute('Location');

                self.$Location = new GeoLocation({
                    country    : Location.attributes.country,
                    city       : Location.attributes.city,
                    zipCode    : Location.attributes.zipCode,
                    street     : Location.attributes.street,
                    houseNumber: Location.attributes.houseNumber
                });

                var countryName = '';

                if (Location.attributes.country) {
                    countryName = QUILocale.get('quiqqer/countries', 'country.' + Location.attributes.country);
                }

                self.$Elm.getElement('.quiqqer-locations-attributes-coordinates-content').set(
                    'html',
                    Mustache.render(template, {
                        countryLabel    : QUILocale.get(lg, 'attribute.country.title'),
                        country         : countryName,
                        cityLabel       : QUILocale.get(lg, 'attribute.city.title'),
                        city            : Location.attributes.city,
                        zipCodeLabel    : QUILocale.get(lg, 'attribute.zipCode.title'),
                        zipCode         : Location.attributes.zipCode,
                        streetLabel     : QUILocale.get(lg, 'attribute.street.title'),
                        street          : Location.attributes.street,
                        houseNumberLabel: QUILocale.get(lg, 'attribute.houseNumber.title'),
                        houseNumber     : Location.attributes.houseNumber,
                        latLabel        : QUILocale.get(lg, 'controls.attributes.coordinates.template.latLabel'),
                        lngLabel        : QUILocale.get(lg, 'controls.attributes.coordinates.template.lngLabel'),
                        tableHeader     : QUILocale.get(lg, 'controls.attributes.coordinates.template.tableHeader'),
                        notationInfo    : QUILocale.get(lg, 'controls.attributes.coordinates.template.notationInfo')
                    })
                );

                var FuncInputChange = function () {
                    if (self.$Map) {
                        self.$showCoordinates();
                    }
                };

                self.$LatInput = self.$Elm.getElement(
                    '.quiqqer-locations-attributes-coordinates-info-coordinates-input-lat'
                );

                self.$LngInput = self.$Elm.getElement(
                    '.quiqqer-locations-attributes-coordinates-info-coordinates-input-lng'
                );

                self.$LatInput.addEvents({
                    change: FuncInputChange
                });

                self.$LngInput.addEvents({
                    change: FuncInputChange
                });

                if (self.$Map) {
                    new QUIButton({
                        textimage: 'fa fa-search',
                        text     : QUILocale.get(lg, 'controls.attributes.coordinates.getbyaddress.btn'),
                        title    : QUILocale.get(lg, 'controls.attributes.coordinates.getbyaddress.btn'),
                        alt      : QUILocale.get(lg, 'controls.attributes.coordinates.getbyaddress.btn'),
                        events   : {
                            onClick: self.$determineCoordinatesByAddress
                        }
                    }).inject(
                        self.$Elm.getElement(
                            '.quiqqer-locations-attributes-coordinates-info-coordinates-btn'
                        )
                    );
                }

                if (self.$Input.value !== '') {
                    var Data = JSON.decode(self.$Input.value);

                    self.$LatInput.value = Data.coordinates.lat;
                    self.$LngInput.value = Data.coordinates.lng;
                    self.$MapLocation    = new GeoLocation(Data.location);

                    var LocationPoint = new GeoPoint({
                        lat: Data.coordinates.lat,
                        lng: Data.coordinates.lng
                    });

                    if (!self.$Map) {
                        return;
                    }

                    self.$Map.showCoordinates(LocationPoint);
                    self.$Map.setCurrentMarker(LocationPoint);
                    self.$Map.setZoom(self.$Map.ZOOM_LEVEL_CLOSE);

                    if (!self.$addressesEqual(self.$MapLocation.getAddress(), self.$Location.getAddress())) {
                        self.$showAddressMismatchWarning(self.$MapLocation.getAddress());
                    }

                    return;
                }

                if (!self.$Location.getAttribute('country') || !self.$Location.getAttribute('city')) {
                    // show Europe
                    self.$Map.showCoordinates(new GeoPoint({
                        lat: 52.5,
                        lng: 13.4
                    }));

                    self.$Map.setZoom(self.$Map.ZOOM_LEVEL_FAR);
                }
            }, 200);
        },

        /**
         * Display a warning if the selected address on the map
         * and the location address mismatch
         *
         * @param {Object} Address - The address selected on the map
         */
        $showAddressMismatchWarning: function (Address) {
            var WarningElm = this.$Elm.getElement(
                '.quiqqer-locations-attributes-coordinates-warning'
            );

            WarningElm.setStyle('display', 'block');

            WarningElm.set(
                'html',
                '<h3>' +
                QUILocale.get(lg, 'controls.attributes.coordinates.address.mismatch.title') +
                '</h3>' +
                '<p>' +
                QUILocale.get(lg, 'controls.attributes.coordinates.popup.content.address.info') +
                '</p>'
            );

            var countryName = '-';

            if (Address.country) {
                countryName = QUILocale.get(
                    'quiqqer/countries', 'country.' +
                    Address.country.toUpperCase()
                );
            }

            var AddressView = {
                city       : '-',
                zipCode    : '-',
                street     : '-',
                houseNumber: '-'
            };

            for (var property in Address) {
                if (!Address.hasOwnProperty(property)) {
                    continue;
                }

                if (Address[property]) {
                    AddressView[property] = Address[property];
                }
            }

            new Element('div', {
                html: Mustache.render(addressTemplate, Object.merge(AddressView, {
                    country         : countryName,
                    countryLabel    : QUILocale.get(lg, 'attribute.country.title'),
                    cityLabel       : QUILocale.get(lg, 'attribute.city.title'),
                    zipCodeLabel    : QUILocale.get(lg, 'attribute.zipCode.title'),
                    streetLabel     : QUILocale.get(lg, 'attribute.street.title'),
                    houseNumberLabel: QUILocale.get(lg, 'attribute.houseNumber.title'),
                    tableHeader     : QUILocale.get(
                        lg,
                        'controls.attributes.coordinates.popup.table.header'
                    )
                }))
            }).inject(
                WarningElm
            );
        },

        /**
         * Hide the address mismatch warning
         */
        $hideAdressMismatchWarning: function () {
            this.$Elm.getElement(
                '.quiqqer-locations-attributes-coordinates-warning'
            ).setStyle(
                'display',
                'none'
            );
        },

        /**
         * Show coordinates on map (based on lat and lng inputs)
         */
        $showCoordinates: function () {
            if (this.$LatInput.value.trim() === '' || this.$LngInput.value.trim() === '') {
                return;
            }

            var self     = this;
            var latValue = this.$LatInput.value.trim();

            latValue = latValue.replace(',', '.');
            latValue = latValue.replace(/[^\d\.\-]/gi, '');

            this.$LatInput.value = latValue;

            var lngValue = this.$LngInput.value.trim();

            lngValue = lngValue.replace(',', '.');
            lngValue = lngValue.replace(/[^\d\.\-]/i, '');

            this.$LngInput.value = lngValue;

            var Point = new GeoPoint({
                lat: latValue,
                lng: lngValue
            });

            this.$Map.showCoordinates(Point);
            this.$Map.setCurrentMarker(Point);
            this.$Map.setZoom(this.$Map.ZOOM_LEVEL_CLOSE);

            Geo.getLocationByCoordinates(Point).then(function (Location) {
                if (!Location) {
                    self.$showAddressMismatchWarning({});
                    return;
                }

                var LocationAddress = Location.getAddress();

                if (!self.$addressesEqual(LocationAddress, self.$Location.getAddress())) {
                    self.$showAddressMismatchWarning(LocationAddress);
                }

                self.$MapLocation = Location;
                self.$setValue();
            });
        },

        /**
         * Try to determine the address by location address
         */
        $determineCoordinatesByAddress: function () {
            var self = this;

            this.$Map.showLocationOnMap(this.$Location).then(function (Coordinates) {
                if (!Coordinates) {
                    new QUIConfirm({
                        maxHeight: 300,
                        maxWidth : 650,
                        autoclose: true,

                        information: QUILocale.get(lg, 'controls.attributes.Coordinates.no_address.info'),
                        title      : QUILocale.get(lg, 'controls.attributes.Coordinates.no_address.title'),
                        texticon   : 'fa fa-map-marker',
                        text       : QUILocale.get(lg, 'controls.attributes.Coordinates.no_address.text'),
                        icon       : 'fa fa-info-circle',

                        cancel_button: false,
                        ok_button    : {
                            text     : QUILocale.get(lg, 'controls.attributes.Coordinates.no_address.ok'),
                            textimage: 'icon-ok fa fa-check'
                        }
                    }).open();

                    return;
                }

                self.$Map.setCurrentMarker(Coordinates);
                self.$Map.setZoom(self.$Map.ZOOM_LEVEL_CLOSE);

                self.$LatInput.value = Coordinates.getLatitude();
                self.$LngInput.value = Coordinates.getLongitude();
                self.$MapLocation    = self.$Location;

                self.$hideAdressMismatchWarning();
                self.$setValue();
            });
        },

        /**
         * Get value
         *
         * @return {Object}
         */
        getValue: function () {

        },

        /**
         * Parse all contact info inputs and set value
         */
        $setValue: function () {
            var Data = {
                coordinates: {
                    lat: this.$LatInput.value,
                    lng: this.$LngInput.value
                },
                location   : this.$MapLocation.toArray()
            };

            this.$Input.value = JSON.encode(Data);
        },

        /**
         * Check if two addresses are equal
         *
         * @param {Object} Addr1
         * @param {Object} Addr2
         * @return {boolean}
         */
        $addressesEqual: function (Addr1, Addr2) {
            return Object.toQueryString(Addr1) === Object.toQueryString(Addr2);
        }
    });
});
