/**
 * @module package/quiqqer/locations/bin/controls/attributes/Image
 * @author www.pcsg.de (Henning Leutz)
 * @author www.pcsg.de (Patrick Müller)
 *
 * @event onChange [imageUrl, this]
 */
define('package/quiqqer/locations/bin/controls/attributes/Image', [

    'qui/QUI',
    'qui/controls/Control',
    'qui/utils/String',
    'controls/projects/project/media/Input',

    'Locale'

], function (QUI, QUIControl, QUIStringUtils, MediaInput, QUILocale) {
    "use strict";

    return new Class({
        Extends: QUIControl,
        Type   : 'package/quiqqer/locations/bin/controls/attributes/Image',

        Binds: [
            '$onImport',
            'openMedia',
            'setValue'
        ],

        options: {
            imagesMediaFolder: false
        },

        initialize: function (options) {
            this.parent(options);

            this.$Input                = null;
            this.$imagesMediaFolder    = false;
            this.$NoMediaFolderInfoElm = null;

            this.addEvents({
                onImport: this.$onImport
            });
        },

        /**
         * event : on import
         */
        $onImport: function () {
            var self = this;

            this.$Input = this.getElm();

            //new Element('span', {
            //    'class': 'field-container-item',
            //    html   : '<span class="fa fa-picture-o"></span>',
            //    styles : {
            //        cursor    : 'pointer',
            //        lineHeight: 30,
            //        textAlign : 'center',
            //        width     : 50
            //    },
            //    events : {
            //        click: this.openMedia
            //    }
            //}).inject(Elm, 'after');
            //
            //Elm.type = 'text';
            //Elm.addClass('field-container-field');

            //this.$Input.type = 'hidden';

            var waitForLocation = setInterval(function () {
                if (self.getAttribute('Location')) {
                    clearInterval(waitForLocation);
                    self.$buildControl();
                }
            }, 200);

        },

        $buildControl: function () {
            var self    = this,
                value   = this.$Input.value,
                fileid  = false,
                project = false;

            var imagesMediaFolder = this.getAttribute('Location').attributes.imagesMediaFolder,
                urlParams         = {};

            if (!imagesMediaFolder) {
                imagesMediaFolder = self.$imagesMediaFolder;
            }

            if (!imagesMediaFolder) {
                this.$Input.disabled = true;

                this.$NoMediaFolderInfoElm = new Element('span', {
                    'class': 'field-container-field',
                    html   : QUILocale.get(
                        'quiqqer/locations',
                        'controls.attributes.image.no.mediafolder'
                    )
                }).inject(this.$Input, 'before');

                var Panel = this.getAttribute('Panel');

                if (!Panel) {
                    return;
                }

                var MediaFolderInput = Panel.getContent().getElement(
                    'input[name="imagesMediaFolder"]'
                );

                if (!MediaFolderInput) {
                    return;
                }

                var MediaFolderContrtol = QUI.Controls.getById(
                    MediaFolderInput.getProperty('data-quiid')
                );

                if (!MediaFolderContrtol) {
                    return;
                }

                MediaFolderContrtol.addEvents({
                    onMediaFolderCreated: function (mediaFolderId) {
                        self.$imagesMediaFolder = mediaFolderId;
                        self.$Input.value       = '';
                        self.$buildControl();
                    }
                });

                return;
            }

            if (this.$NoMediaFolderInfoElm) {
                this.$NoMediaFolderInfoElm.destroy();
                this.$NoMediaFolderInfoElm = null;
            }

            if (value === '' && imagesMediaFolder) {
                urlParams = QUIStringUtils.getUrlParams(imagesMediaFolder);
            } else if (value !== '') {
                urlParams = QUIStringUtils.getUrlParams(value);
            }

            if ("id" in urlParams) {
                fileid = urlParams.id;
            }

            if ("project" in urlParams) {
                project = urlParams.project;
            }

            var MediaInputControl = new MediaInput({
                fileid          : fileid,
                breadcrumb      : false,     // you can specified if the breadcrumb is shown or not
                selectable_types: ['image']
            }, this.$Input);

            MediaInputControl.setProject(project);
            MediaInputControl.create();
        },

        /**
         * Set value
         *
         * @param {string} value - QUIQQER image URL
         */
        setValue: function (value) {
            if (value) {
                this.$Input.value = value;
            }
        },

        /**
         * Return the current value
         *
         * @returns {String}
         */
        getValue: function () {
            return this.$Input.value;
        }
    });
});
