/**
 * Control for managing location media folders
 *
 * @module package/quiqqer/locations/bin/controls/attributes/MediaFolder
 * @author www.pcsg.de (Patrick Müller)
 *
 * @event onMediaFolderCreated [mediaFolderUrl, this]
 */
define('package/quiqqer/locations/bin/controls/attributes/MediaFolder', [

    'qui/controls/Control',
    'qui/controls/buttons/Button',
    'qui/controls/loader/Loader',

    'controls/projects/project/media/FolderViewer',

    'Ajax',
    'Locale',

    'css!package/quiqqer/locations/bin/controls/attributes/MediaFolder.css'

], function (QUIControl, QUIButton, QUILoader, FolderViewer, QUIAjax, QUILocale) {
    "use strict";

    var lg = 'quiqqer/locations';

    return new Class({
        Extends: QUIControl,
        Type   : 'package/quiqqer/locations/bin/controls/attributes/MediaFolder',

        Binds: [
            '$onImport',
            '$load',
            '$createMediaFolder'
        ],

        initialize: function (options) {
            this.parent(options);

            this.$Input          = null;
            this.$mediaFolderUrl = false;
            this.Loader          = new QUILoader();

            this.addEvents({
                onImport: this.$onImport
            });
        },

        /**
         * Create Control
         *
         * @return {HTMLElement}
         */
        create: function () {
            this.$Elm = new Element('div', {
                'class': 'quiqqer-locations-attributes-mediafolder'
            });

            this.Loader.inject(this.$Elm);

            return this.$Elm;
        },

        /**
         * Event: onImport
         */
        $onImport: function () {
            this.$Input      = this.getElm();
            this.$Input.type = 'hidden';
            //this.$Input.setStyle('display', 'none');

            if (this.$Input.value.trim() !== '') {
                this.$mediaFolderUrl = this.$Input.value;
            }

            var Elm = this.create().inject(this.$Input, 'after');

            if (this.$Input.getParent('.field-container')) {
                var Container = new Element('div', {
                    'class': 'field-container-field'
                });

                Container.wraps(Elm);
            }

            this.$load();
        },

        /**
         * Load control data
         */
        $load: function () {
            var self = this;

            // if no folder has been chosen yet, show "create media folder" button
            if (!this.$mediaFolderUrl) {
                new QUIButton({
                    textimage: 'fa fa-folder',
                    text     : QUILocale.get(lg, 'controls.attributes.mediafolder.create.btn.text'),
                    title    : QUILocale.get(lg, 'controls.attributes.mediafolder.create.btn.text'),
                    alt      : QUILocale.get(lg, 'controls.attributes.mediafolder.create.btn.text'),
                    events   : {
                        onClick: function (Btn) {
                            self.Loader.show();
                            Btn.disable();

                            self.$createMediaFolder().then(function (mediaFolderUrl) {
                                self.Loader.hide();

                                if (mediaFolderUrl) {
                                    self.$Input.value    = mediaFolderUrl;
                                    self.$mediaFolderUrl = mediaFolderUrl;
                                    self.fireEvent('mediaFolderCreated', [mediaFolderUrl, self]);
                                }

                                Btn.destroy();
                                self.$load();
                            });
                        }
                    }
                }).inject(this.$Elm);

                return;
            }

            var Viewer = new FolderViewer({
                'class'     : 'quiqqer-locations-attributes-mediafolder-viewer',
                folderUrl   : this.$mediaFolderUrl,
                filetype    : ['image'],
                autoactivate: true
            }).inject(this.$Elm);

            (function() {
                Viewer.getElm().setStyle('height', self.$Elm.getHeight());
            }.delay(200));
        },

        /**
         * Create new media folder
         */
        $createMediaFolder: function () {
            var self = this;

            return new Promise(function (resolve, reject) {
                QUIAjax.post('package_quiqqer_locations_ajax_attributes_MediaFolder_create', resolve, {
                    'package' : 'quiqqer/locations',
                    onError   : reject,
                    locationId: self.getAttribute('Location').id
                });
            });
        }
    });
});