/**
 * One entry for an opening time
 *
 * @module package/quiqqer/locations/bin/controls/attributes/OpeningHours/OpeningHoursEntry
 * @author www.pcsg.de (Patrick Müller)
 *
 * @event onChange [this] - fires when any value is changed
 */
define('package/quiqqer/locations/bin/controls/attributes/OpeningHours/OpeningHoursEntry', [

    'qui/QUI',
    'qui/controls/Control',
    'qui/controls/buttons/Button',
    'qui/controls/buttons/Select',

    'Locale',

    'css!package/quiqqer/locations/bin/controls/attributes/OpeningHours/OpeningHoursEntry.css'

], function (QUI, QUIControl, QUIButton, QUISelect, QUILocale) {
    "use strict";

    var lg = 'quiqqer/locations';

    return new Class({

        Extends: QUIControl,
        Type   : 'package/quiqqer/locations/bin/controls/attributes/OpeningHours/OpeningHoursEntry',

        Binds: [
            'setValue',
            'getValue'
        ],

        initialize: function (options) {
            this.parent(options);

            this.$TypeSelect = null;
        },

        /**
         * create the domnode element
         *
         * @return {HTMLDivElement}
         */
        create: function () {
            var self = this;

            this.$Elm = new Element('div', {
                'class': 'quiqqer-locations-openinghours-entry',
                html   : '<div class="quiqqer-locations-openinghours-entry-select"></div>' +
                    '<div class="quiqqer-locations-openinghours-entry-times">' +
                    '<label>' +
                    '<span>' +
                    QUILocale.get(lg, 'controls.attributes.openinghoursentry.label.from') +
                    '</span>' +
                    '<input type="time"' +
                    ' class="time-input quiqqer-locations-openinghours-entry-times-from"' +
                    ' placeholder="08:00">' +
                    '</label>' +
                    '<label>' +
                    '<span>' +
                    QUILocale.get(lg, 'controls.attributes.openinghoursentry.label.to') +
                    '</span>' +
                    '<input type="time"' +
                    ' class="time-input quiqqer-locations-openinghours-entry-times-to"' +
                    ' placeholder="17:00">' +
                    '</label>' +
                    '</div>' +
                    '<div class="quiqqer-locations-openinghours-entry-options">' +
                    '<label>' +
                    '<span>' + QUILocale.get(lg, 'controls.attributes.openinghoursentry.label.closed24') + '</span>' +
                    '<input type="checkbox" class="quiqqer-locations-openinghours-entry-options-closed24">' +
                    '</label>' +
                    '<label>' +
                    '<span>' + QUILocale.get(lg, 'controls.attributes.openinghoursentry.label.open24') + '</span>' +
                    '<input type="checkbox" class="quiqqer-locations-openinghours-entry-options-open24">' +
                    '</label>' +
                    '</div>'
            });

            this.$TypeSelect = new Element('select', {
                name  : 'type',
                events: {
                    change: function () {
                        self.fireEvent('change', [self]);
                    }
                }
            }).inject(
                this.$Elm.getElement('.quiqqer-locations-openinghours-entry-select')
            );

            var typeEntries = [
                'motofr', 'motosa', 'sasu', 'mo', 'tu', 'we', 'th', 'fr', 'sa', 'su',
                'everyday'
            ];

            typeEntries.each(function (value) {
                new Element('option', {
                    html : QUILocale.get(lg, 'controls.attributes.openinghoursentry.entry.' + value),
                    value: value
                }).inject(self.$TypeSelect);
            });

            this.$TypeSelect.value = 'motofr';

            // events
            this.$Elm.getElement(
                '.quiqqer-locations-openinghours-entry-options-closed24'
            ).addEvents({
                click: function (event) {
                    var on = event.target.checked;

                    var Open24Elm = self.$Elm.getElement(
                        '.quiqqer-locations-openinghours-entry-options-open24'
                    );

                    self.$Elm.getElements(
                        '.quiqqer-locations-openinghours-entry-times input'
                    ).each(function (TimeInput) {
                        if (on) {
                            TimeInput.value    = null;
                            TimeInput.disabled = true;
                            Open24Elm.disabled = true;
                        } else {
                            TimeInput.disabled = false;
                            Open24Elm.disabled = false;
                        }
                    });

                    self.fireEvent('change', [self]);
                }
            });

            this.$Elm.getElement(
                '.quiqqer-locations-openinghours-entry-options-open24'
            ).addEvents({
                click: function (event) {
                    var FromElm = self.$Elm.getElement(
                        '.quiqqer-locations-openinghours-entry-times-from'
                    );

                    var ToElm = self.$Elm.getElement(
                        '.quiqqer-locations-openinghours-entry-times-to'
                    );

                    var Closed24Elm = self.$Elm.getElement(
                        '.quiqqer-locations-openinghours-entry-options-closed24'
                    );

                    if (event.target.checked) {
                        FromElm.value        = '00:00';
                        ToElm.value          = '23:59';
                        FromElm.disabled     = true;
                        ToElm.disabled       = true;
                        Closed24Elm.disabled = true;
                    } else {
                        FromElm.disabled     = false;
                        ToElm.disabled       = false;
                        Closed24Elm.disabled = false;
                    }

                    self.fireEvent('change', [self]);
                }
            });

            this.$Elm.getElements('.quiqqer-locations-openinghours-entry-times input').addEvents({
                change: function (event) {
                    self.fireEvent('change', [self]);
                }
            });

            return this.$Elm;
        },

        /**
         * Get current value
         *
         * @return {Object}
         */
        getValue: function () {
            var fromValue = this.$Elm.getElement('.quiqqer-locations-openinghours-entry-times-from').value;
            var toValue   = this.$Elm.getElement('.quiqqer-locations-openinghours-entry-times-to').value;

            if (fromValue) {
                fromValue = fromValue.replace(':', '');
            } else {
                fromValue = false;
            }

            if (toValue) {
                toValue = toValue.replace(':', '');
            } else {
                toValue = false;
            }

            return {
                range   : this.$TypeSelect.value,
                closed24: this.$Elm.getElement('.quiqqer-locations-openinghours-entry-options-closed24').checked,
                open24  : this.$Elm.getElement('.quiqqer-locations-openinghours-entry-options-open24').checked,
                from    : fromValue,
                to      : toValue
            }
        },

        /**
         * Set value
         *
         * @param {Object} Data
         */
        setValue: function (Data) {
            this.$TypeSelect.value = Data.range;

            if (Data.closed24) {
                this.$Elm.getElement('.quiqqer-locations-openinghours-entry-options-closed24').click();
            }

            if (Data.open24) {
                this.$Elm.getElement('.quiqqer-locations-openinghours-entry-options-open24').click();
            }

            if (Data.from) {
                var fromValue = Data.from.substr(0, 2) + ':' + Data.from.substr(2, 2);

                this.$Elm.getElement('.quiqqer-locations-openinghours-entry-times-from').value = fromValue;
            }

            if (Data.to) {
                var toValue = Data.to.substr(0, 2) + ':' + Data.to.substr(2, 2);

                this.$Elm.getElement('.quiqqer-locations-openinghours-entry-times-to').value = toValue;
            }
        }
    });
});
