/**
 * List of locations and CRUD functions
 *
 * @module package/quiqqer/locations/bin/controls/locations/Panel
 * @author www.pcsg.de (Patrick Müller)
 */
define('package/quiqqer/locations/bin/controls/locations/Panel', [

    'qui/controls/desktop/Panel',
    'qui/controls/buttons/Separator',
    'qui/controls/buttons/Button',
    'qui/controls/loader/Loader',
    'qui/controls/windows/Popup',
    'qui/controls/windows/Confirm',
    'controls/grid/Grid',

    'package/quiqqer/locations/bin/Locations',

    'Ajax',
    'Locale',

    'css!package/quiqqer/locations/bin/controls/locations/Panel.css'

], function (QUIPanel, QUISeparator, QUIButton, QUILoader, QUIPopup, QUIConfirm,
             Grid, Locations, Ajax, QUILocale) {
    "use strict";

    var lg = 'quiqqer/locations';

    return new Class({

        Extends: QUIPanel,
        Type   : 'package/quiqqer/locations/bin/controls/locations/Panel',

        Binds: [
            '$onInject',
            '$onRefresh',
            '$onCreate',
            '$onResize',
            'refresh',
            'createLocation',
            'deleteLocations',
            'editLocation',
            '$listRefresh',
            '$addRemoveSearchBtn'
        ],

        initialize: function (options) {
            this.setAttribute('title', QUILocale.get(lg, 'controls.locations.panel.title'));

            this.parent(options);

            this.addEvents({
                onRefresh: this.$onRefresh,
                onCreate : this.$onCreate,
                onResize : this.$onResize
            });

            this.Loader         = new QUILoader();
            this.$GridContainer = null;
            this.$Grid          = null;
            this.$SearchParams  = {};
            this.$SearchInput   = null;
            this.$removeBtn     = false;

            // debugging
            //require([
            //    'package/quiqqer/geoutils/bin/Geo',
            //    'package/quiqqer/geoutils/bin/classes/GeoPoint',
            //    'package/quiqqer/geoutils/bin/classes/GeoLocation'
            //], function(Geo, Point, Location) {
            //
            //    var L = new Location();
            //    L.setPosition(new Point({
            //        lat: 51.20015,
            //        lng: 7.08606
            //    }));
            //
            //    var L2 = new Location();
            //    L2.setPosition(new Point({
            //        lat: 51.17244,
            //        lng: 7.08351
            //    }));
            //
            //    var list = [L, L2];
            //
            //    Geo.getSuggestionListBySearchString(
            //        '426',
            //        'de'
            //    ).then(function(r) {
            //        console.log(r);
            //    });
            //
            //    //var M = new Module({
            //    //    events: {
            //    //        onLoaded: function() {
            //    //            //var PointA = new Point({
            //    //            //    lat: 51.20015,
            //    //            //    lng: 7.08606
            //    //            //});
            //    //
            //    //            var Center = new Point({
            //    //                lat: 51.14891,
            //    //                lng: 7.10671
            //    //            });
            //    //
            //    //            M.proximitySearchCustomList(Center, list).then(function(result) {
            //    //                console.log(result[0].getAttribute('distance'));
            //    //            });
            //    //        }
            //    //    }
            //    //});
            //});
        },

        /**
         * event on DOMElement creation
         */
        $onCreate: function () {
            var self    = this,
                Content = this.getContent();

            Content.setStyles({
                padding: 0
            });

            // buttons
            this.addButton({
                name     : 'add',
                text     : QUILocale.get(lg, 'controls.locations.panel.btn.add'),
                textimage: 'fa fa-plus',
                events   : {
                    onClick: this.createLocation
                }
            });

            this.addButton({
                name     : 'edit',
                text     : QUILocale.get(lg, 'controls.locations.panel.btn.edit'),
                textimage: 'fa fa-edit',
                events   : {
                    onClick: function () {
                        self.$openLocationPanel(
                            self.$Grid.getSelectedData()[0].id
                        );
                    }
                }
            });

            this.addButton(new QUISeparator());

            this.addButton({
                name     : 'delete',
                text     : QUILocale.get(lg, 'controls.locations.panel.btn.delete'),
                textimage: 'fa fa-trash',
                events   : {
                    onClick: self.deleteLocations
                }
            });

            this.$SearchInput = new Element('div', {
                'class': 'quiqqer-locations-panel-search-input',
                html   : '<input type="text" data-mode="search"/><span class="fa fa-search"></span>'
            });

            this.$SearchInput.getElement('input').setProperty(
                'placeholder',
                QUILocale.get(lg, 'controls.locations.panel.searchinput.placeholder')
            );

            this.$SearchInput.getElement('input').addEvents({
                keyup: function (event) {
                    if (event.code === 13) {
                        self.$SearchParams.searchterm = event.target.value.trim();
                        self.refresh();
                    }
                }
            });

            this.$SearchInput.getElement('span').addEvents({
                click: function () {
                    var Input = self.$SearchInput.getElement('input');

                    if (Input.getProperty('data-mode') === 'refresh') {
                        Input.value = '';
                        Input.focus();
                        self.$SearchParams.searchterm = '';
                        self.refresh();

                        return;
                    }

                    if (Input.value.trim() === '') {
                        Input.value = '';
                        Input.focus();

                        return;
                    }

                    self.$SearchParams.searchterm = Input.value.trim();
                    self.refresh();
                }
            });

            this.addButton(this.$SearchInput);

            // content
            this.$GridContainer = new Element('div', {
                'class': 'quiqqer-locations-panel-container'
            }).inject(Content);

            this.$GridFX = moofx(this.$GridContainer);

            var GridContainer = new Element('div', {
                'class': 'quiqqer-locations-panel-grid'
            }).inject(this.$GridContainer);

            var columns = [{
                header   : QUILocale.get('quiqqer/system', 'id'),
                dataIndex: 'id',
                dataType : 'number',
                width    : 50
            }, {
                header   : QUILocale.get('quiqqer/system', 'status'),
                dataIndex: 'active',
                dataType : 'node',
                width    : 50
            }];

            this.Loader.show();

            Locations.getListColumns().then(function (listColumns) {
                for (var i = 0, len = listColumns.length; i < len; i++) {
                    var Column = listColumns[i];

                    columns.push({
                        header   : Column.header,
                        dataIndex: Column.name,
                        dataType : 'node',
                        width    : 175
                    });
                }

                self.$Grid = new Grid(GridContainer, {
                    pagination       : true,
                    serverSort       : true,
                    selectable       : true,
                    multipleSelection: true,
                    columnModel      : columns
                });

                self.$Grid.addEvents({
                    onDblClick: function () {
                        self.$openLocationPanel(
                            self.$Grid.getSelectedData()[0].id
                        );
                    },
                    onClick   : function () {
                        self.getButtons('edit').enable();
                        self.getButtons('delete').enable();
                    },
                    onRefresh : self.$listRefresh
                });

                self.Loader.hide();
                self.resize();
                self.refresh();
            });

        },

        /**
         * event: onResize
         */
        $onResize: function () {
            // workaround - force shows text on button bar buttons
            this.getButtonBar()
                .getElm()
                .getElement('.qui-toolbar-tabs')
                .removeClass('qui-toolbar--mobile');

            if (this.$GridContainer && this.$Grid) {
                var size = this.$GridContainer.getSize();

                this.$Grid.setHeight(size.y);
                this.$Grid.resize();
            }
        },

        /**
         * refresh grid
         *
         * @param {Object} Grid
         */
        $listRefresh: function (Grid) {
            var self = this;

            this.Loader.show();

            var GridParams = {
                sortOn : Grid.getAttribute('sortOn'),
                sortBy : Grid.getAttribute('sortBy'),
                perPage: Grid.getAttribute('perPage'),
                page   : Grid.getAttribute('page')
            };

            this.getButtons('delete').disable();
            this.getButtons('edit').disable();

            Locations.search(
                Object.merge(GridParams, this.$SearchParams)
            ).then(function (gridData) {
                self.$setGridData(gridData);

                var SearchIcon = self.$SearchInput.getElement('span');

                if ("searchterm" in self.$SearchParams &&
                    self.$SearchParams.searchterm !== '') {

                    SearchIcon.removeClass('fa-search');
                    SearchIcon.addClass('fa-times');
                    self.$SearchInput.getElement('input').setProperty('data-mode', 'refresh');
                } else {
                    SearchIcon.removeClass('fa-times');
                    SearchIcon.addClass('fa-search');
                    self.$SearchInput.getElement('input').setProperty('data-mode', 'search');
                }

                self.Loader.hide();
            });
        },

        /**
         * refresh the password list
         */
        refresh: function () {
            if (this.$Grid) {
                this.$Grid.refresh();
            }
        },

        /**
         * Set data to grid
         *
         * @param {object} GridData
         */
        $setGridData: function (GridData) {
            for (var i = 0, len = GridData.data.length; i < len; i++) {
                var Row = GridData.data[i];

                if (Row.active == 1) {
                    Row.active = new Element('span', {
                        'class': 'fa fa-check',
                        title  : QUILocale.get(lg, 'controls.locations.panel.tbl.status.active'),
                        alt    : QUILocale.get(lg, 'controls.locations.panel.tbl.status.active')
                    });
                } else {
                    Row.active = new Element('span', {
                        'class': 'fa fa-close',
                        title  : QUILocale.get(lg, 'controls.locations.panel.tbl.status.inactive'),
                        alt    : QUILocale.get(lg, 'controls.locations.panel.tbl.status.inactive')
                    });
                }

                for (var attribute in Row) {
                    if (!Row.hasOwnProperty(attribute)) {
                        continue;
                    }

                    switch (attribute) {
                        case 'id':
                        case 'active':
                            continue;
                    }

                    if (Row[attribute] === "1") {
                        Row[attribute] = new Element('span', {
                            'class': 'fa fa-check'
                        });
                    } else if (Row[attribute] === "0") {
                        Row[attribute] = new Element('span', {
                            'class': 'fa fa-close'
                        });
                    } else {
                        Row[attribute] = new Element('span', {
                            html : Row[attribute] ? Row[attribute] : '-',
                            title: Row[attribute] ? Row[attribute] : '-'
                        });
                    }
                }
            }

            this.$Grid.setData(GridData);
        },

        /**
         * Create new location
         */
        createLocation: function () {
            var self = this;

            var FuncSubmit = function () {
                var Input = Popup.getContent()
                    .getElement(
                        '.quiqqer-locations-panel-create-input'
                    );

                var name = Input.value.trim();

                if (name === '') {
                    Input.value = '';
                    Input.focus();
                    return;
                }

                Popup.Loader.show();

                var lang = QUILocale.getCurrent();

                var nameVal = {};

                nameVal[lang] = name;

                Locations.createLocation(JSON.encode(nameVal)).then(function (newLocationId) {
                    if (!newLocationId) {
                        Popup.Loader.hide();
                        return;
                    }

                    self.refresh();
                    Popup.close();

                    self.$openLocationPanel(newLocationId);
                });
            };

            // open popup
            var Popup = new QUIPopup({
                icon       : 'fa fa-plus',
                title      : QUILocale.get(
                    lg, 'controls.locations.panel.create.popup.title'
                ),
                maxHeight  : 200,
                maxWidth   : 450,
                events     : {
                    onOpen: function () {
                        var Input = Popup.getContent()
                            .getElement(
                                '.quiqqer-locations-panel-create-input'
                            );

                        Input.addEvents({
                            keyup: function (event) {
                                if (event.code === 13) {
                                    FuncSubmit();
                                    Input.blur();
                                }
                            }
                        });

                        Input.focus();
                    }
                },
                closeButton: true,
                content    : '<label class="quiqqer-locations-panel-create-label">' +
                    '<span>' + QUILocale.get(lg, 'controls.locations.panel.create.popup.info.title') + '</span>' +
                    '<input type="text" class="quiqqer-locations-panel-create-input"/>' +
                    '</label>'
            });

            Popup.open();

            Popup.addButton(new QUIButton({
                text  : QUILocale.get(lg, 'controls.locations.panel.create.popup.confirm.btn.text'),
                alt   : QUILocale.get(lg, 'controls.locations.panel.create.popup.confirm.btn'),
                title : QUILocale.get(lg, 'controls.locations.panel.create.popup.confirm.btn'),
                events: {
                    onClick: FuncSubmit
                }
            }));
        },

        /**
         * Opens a panel for a single location
         *
         * @param {number} locationId
         */
        $openLocationPanel: function (locationId) {
            require([
                'package/quiqqer/locations/bin/controls/locations/Location',
                'utils/Panels'
            ], function (LocationPanel, Utils) {
                Utils.openPanelInTasks(new LocationPanel({
                    locationId: locationId,
                    '#id'     : 'quiqqer_location_' + locationId
                }));
            });
        },

        /**
         * Deletes one or multiple location(s) that are selected in the grid table
         */
        deleteLocations: function () {
            var self                = this;
            var deleteLocationsData = [];
            var deleteLocationsIds  = [];
            var rows                = this.$Grid.getSelectedData();

            for (var i = 0, len = rows.length; i < len; i++) {
                deleteLocationsData.push(
                    rows[i].name.innerHTML + ' (ID: #' + rows[i].id + ')'
                );

                deleteLocationsIds.push(rows[i].id);
            }

            // open popup
            var Popup = new QUIConfirm({
                'maxHeight': 300,
                'autoclose': true,

                'information': QUILocale.get(
                    lg,
                    'controls.locations.panel.delete.popup.info', {
                        locations: deleteLocationsData.join('<br/>')
                    }
                ),
                'title'      : QUILocale.get(lg, 'controls.locations.panel.delete.popup.title'),
                'texticon'   : 'fa fa-trash',
                'icon'       : 'fa fa-trash',

                cancel_button: {
                    text     : false,
                    textimage: 'icon-remove fa fa-remove'
                },
                ok_button    : {
                    text     : false,
                    textimage: 'icon-ok fa fa-check'
                },
                events       : {
                    onSubmit: function () {
                        Popup.Loader.show();

                        Locations.deleteLocations(deleteLocationsIds).then(function (result) {
                            if (!result) {
                                Popup.Loader.hide();
                                return;
                            }

                            Popup.close();
                            self.refresh();
                        });
                    }
                }
            });

            Popup.open();
        },

        /**
         * Opens password view
         *
         * @param {number} passwordId
         */
        editLocation: function (passwordId) {
            // @todo
        },

        /**
         * Add button that removes search
         */
        $addRemoveSearchBtn: function () {
            var self = this;

            if (this.$removeBtn) {
                return;
            }

            this.$removeBtn = true;

            this.addButton({
                name     : 'searchremove',
                text     : QUILocale.get(lg, 'controls.locations.panel.btn.searchremove'),
                textimage: 'fa fa-times',
                styles   : {
                    float: 'right'
                },
                events   : {
                    onClick: function (Btn) {
                        self.$SearchParams = {};
                        self.$Grid.refresh();
                        Btn.destroy();
                        self.$removeBtn = false;
                    }
                }
            });
        }
    });
});