/**
 * List of location categories and CRUD functions
 *
 * @module package/quiqqer/locations/bin/controls/locationsapis/Panel
 * @author www.pcsg.de (Patrick Müller)
 */
define('package/quiqqer/locations/bin/controls/locationsapis/Panel', [

    'qui/controls/desktop/Panel',
    'qui/controls/buttons/Separator',
    'qui/controls/buttons/Button',
    'qui/controls/loader/Loader',
    'qui/controls/windows/Popup',
    'qui/controls/windows/Confirm',
    'controls/grid/Grid',

    'package/quiqqer/cron/bin/controls/CronTime',

    'Ajax',
    'Locale',

    'css!package/quiqqer/locations/bin/controls/locationsapis/Panel.css'

], function (QUIPanel, QUISeparator, QUIButton, QUILoader, QUIPopup, QUIConfirm,
             Grid, CronTime, QUIAjax, QUILocale) {
    "use strict";

    var lg = 'quiqqer/locations';

    return new Class({

        Extends: QUIPanel,
        Type   : 'package/quiqqer/locations/bin/controls/locationsapis/Panel',

        Binds: [
            '$onInject',
            '$onRefresh',
            '$onCreate',
            '$onResize',
            'refresh',
            'createCategory',
            'deleteCategories',
            'editLocation',
            '$listRefresh',
            '$addRemoveSearchBtn',
            'setExecTime',
            'execute'
        ],

        initialize: function (options) {
            this.setAttribute('title', QUILocale.get(lg, 'controls.locationsapis.panel.title'));

            this.parent(options);

            this.addEvents({
                onRefresh: this.$onRefresh,
                onCreate : this.$onCreate,
                onResize : this.$onResize
            });

            this.Loader         = new QUILoader();
            this.$GridContainer = null;
            this.$Grid          = null;
        },

        /**
         * event on DOMElement creation
         */
        $onCreate: function () {
            var self    = this,
                Content = this.getContent();

            Content.setStyles({
                padding: 0
            });

            // buttons
            this.addButton({
                name     : 'edit',
                text     : QUILocale.get(lg, 'controls.locationsapis.panel.btn.edit'),
                textimage: 'fa fa-edit',
                events   : {
                    onClick: function () {
                        self.$openApiSheet(
                            self.$Grid.getSelectedData()[0].jsControl
                        );
                    }
                }
            });

            this.addButton({
                name     : 'timetable',
                text     : QUILocale.get(lg, 'controls.locationsapis.panel.btn.timetable'),
                textimage: 'fa fa-clock-o',
                events   : {
                    onClick: this.setExecTime
                }
            });

            this.addButton(new QUISeparator());

            this.addButton({
                name     : 'execute',
                text     : QUILocale.get(lg, 'controls.locationsapis.panel.btn.execute'),
                textimage: 'fa fa-play-circle-o',
                events   : {
                    onClick: this.execute
                }
            });

            // content
            this.$GridContainer = new Element('div', {
                'class': 'quiqqer-locations-apis-panel-container'
            }).inject(Content);

            this.$GridFX = moofx(this.$GridContainer);

            var GridContainer = new Element('div', {
                'class': 'quiqqer-locations-apis-panel-grid'
            }).inject(this.$GridContainer);

            var columns = [{
                header   : QUILocale.get('quiqqer/system', 'id'),
                dataIndex: 'id',
                dataType : 'number',
                width    : 50
            }, {
                header   : QUILocale.get(lg, 'controls.locationsapis.panel.table.header.title'),
                dataIndex: 'title',
                dataType : 'string',
                width    : 200
            }, {
                header   : QUILocale.get(lg, 'controls.locationsapis.panel.table.header.description'),
                dataIndex: 'description',
                dataType : 'string',
                width    : 450
            }, {
                header   : QUILocale.get(lg, 'controls.locationsapis.panel.table.header.source_package'),
                dataIndex: 'source_package',
                dataType : 'string',
                width    : 150
            }, {
                dataIndex: 'jsControl',
                dataType : 'integer',
                hidden   : true
            }, {
                dataIndex: 'execTime',
                dataType : 'string',
                hidden   : true
            }];

            this.$Grid = new Grid(GridContainer, {
                pagination       : false,
                serverSort       : false,
                selectable       : true,
                multipleSelection: false,
                columnModel      : columns
            });

            this.$Grid.addEvents({
                onDblClick: function () {
                    self.$openApiSheet(
                        self.$Grid.getSelectedData()[0].jsControl
                    );
                },
                onClick   : function () {
                    var jsControl = self.$Grid.getSelectedData()[0].jsControl;

                    if (jsControl.length) {
                        self.getButtons('edit').enable();
                    } else {
                        self.getButtons('edit').disable();
                    }
                },
                onRefresh : self.$listRefresh
            });

            this.Loader.hide();
            this.resize();
            this.refresh();
        },

        /**
         * event: onResize
         */
        $onResize: function () {
            var size = this.$GridContainer.getSize();

            // workaround - force shows text on button bar buttons
            //this.getButtonBar()
            //    .getElm()
            //    .getElement('.qui-toolbar-tabs')
            //    .removeClass('qui-toolbar--mobile');

            this.$Grid.setHeight(size.y);
            this.$Grid.resize();
        },

        /**
         * Refresh category list
         *
         * @return {Promise}
         */
        refresh: function () {
            var self = this;

            this.Loader.show();

            return new Promise(function (resolve, reject) {
                QUIAjax.get('package_quiqqer_locations_ajax_locationsapis_getList', function (apiList) {
                    self.$Grid.setData({
                        data : apiList,
                        page : 1,
                        total: 1
                    });

                    self.Loader.hide();
                    resolve();
                }, {
                    'package': 'quiqqer/locations',
                    onError  : reject
                });
            });
        },

        /**
         * Opens a sheet for to edit a single location api
         *
         * @param {string} jsControl - path to settings control
         */
        $openApiSheet: function (jsControl) {
            var self = this;

            this.Loader.show();

            this.createSheet({
                title : QUILocale.get(lg, 'controls.locationsapis.panel.edit.title'),
                events: {
                    onShow : function (Sheet) {
                        Sheet.getContent().setStyle('padding', 20);

                        require([
                            jsControl
                        ], function (ApiControl) {
                            self.Loader.hide();

                            new ApiControl({
                                Container: Sheet
                            }).inject(Sheet.getContent());
                        });

                        Sheet.addButton(
                            new QUIButton({
                                text     : QUILocale.get('quiqqer/system', 'save'),
                                textimage: 'fa fa-save',
                                events   : {
                                    onClick: function () {
                                        Sheet.fireEvent('save');
                                        Sheet.destroy();
                                    }
                                }
                            })
                        );
                    },
                    onClose: function (Sheet) {
                        Sheet.destroy();
                    }
                }
            }).show();
        },

        /**
         * Execute selected api module
         */
        execute: function () {
            var self = this;

            this.Loader.show();

            QUIAjax.post(
                'package_quiqqer_locations_ajax_locationsapis_execute',
                function (success) {
                    self.Loader.hide();
                },
                {
                    'package': 'quiqqer/locations',
                    id       : this.$Grid.getSelectedData()[0].id
                }
            );
        },

        /**
         * Set execution time for an API
         */
        setExecTime: function () {
            var self            = this;
            var Api             = this.$Grid.getSelectedData()[0];
            var CronTimeControl = null,
                Input           = null;
            var ExecTime        = JSON.decode(Api.execTime);

            // open popup
            var Popup = new QUIConfirm({
                'maxHeight': 325,
                maxWidth   : 675,
                'autoclose': true,

                'information': QUILocale.get(
                    lg,
                    'controls.locationsapis.panel.exectime.description', {
                        apiTitle: Api.title,
                        apiId   : Api.id
                    }
                ),
                'title'      : QUILocale.get(lg, 'controls.locationsapis.panel.exectime.title'),
                'texticon'   : 'fa fa-clock-o',
                'icon'       : 'fa fa-clock-o',

                cancel_button: {
                    text     : false,
                    textimage: 'icon-remove fa fa-remove'
                },
                ok_button    : {
                    text     : false,
                    textimage: 'icon-ok fa fa-check'
                },
                events       : {
                    onOpen  : function () {
                        var Body           = Popup.getContent().getElement('.textbody');
                        var ExecTimeValues = ExecTime.time;

                        CronTimeControl = new CronTime().inject(Body);

                        CronTimeControl.setValue(
                            ExecTimeValues.minute,
                            ExecTimeValues.hour,
                            ExecTimeValues.day,
                            ExecTimeValues.month,
                            ExecTimeValues.dayOfWeek
                        );

                        var Label = new Element('label', {
                            'class': 'quiqqer-locations-apis-panel-crontime-label',
                            html   : '<span>' +
                            QUILocale.get(lg, 'controls.locationsapis.panel.exectime.enable') +
                            '</span>'
                        }).inject(CronTimeControl.getElm(), 'before');

                        Input = new Element('input', {
                            type  : 'checkbox',
                            events: {
                                change: function (event) {
                                    if (event.target.checked) {
                                        ExecTime.enabled = false;
                                        CronTimeControl.getElm().setStyle('display', 'none');

                                        return;
                                    }

                                    ExecTime.enabled = true;
                                    CronTimeControl.getElm().setStyle('display', '');
                                }
                            }
                        }).inject(Label);

                        if (!ExecTime.enabled) {
                            Input.click();
                        }

                        Popup.getContent().getElement(
                            '.textbody h2'
                        ).setStyle('display', 'none');

                        CronTimeControl.getElm().addClass(
                            'quiqqer-locations-apis-panel-crontime'
                        );
                    },
                    onSubmit: function () {
                        Popup.Loader.show();

                        QUIAjax.post(
                            'package_quiqqer_locations_ajax_locationsapis_updateExecTime',
                            function (success) {
                                Popup.Loader.hide();

                                if (!success) {
                                    return;
                                }

                                Popup.close();
                                self.refresh();
                            },
                            {
                                'package': 'quiqqer/locations',
                                id       : Api.id,
                                execTime : JSON.encode({
                                    enabled: ExecTime.enabled,
                                    time   : CronTimeControl.getValue()
                                })
                            }
                        );
                    }
                }
            });

            Popup.open();
        }
    });
});