/**
 * Select which columns are shown in the locations overview list
 *
 * @module package/quiqqer/locations/bin/controls/settings/ListColumns
 * @author www.pcsg.de (Patrick Müller)
 */
define('package/quiqqer/locations/bin/controls/settings/ListColumns', [

    'qui/controls/Control',
    'package/quiqqer/locations/bin/Locations',

    'css!package/quiqqer/locations/bin/controls/settings/ListColumns.css'

], function (QUIControl, Locations) {
    "use strict";

    return new Class({
        Extends: QUIControl,
        Type   : 'package/quiqqer/locations/bin/controls/settings/ListColumns',

        Binds: [
            '$onImport',
            '$setSettings'
        ],

        options: {},

        initialize: function (options) {
            this.parent(options);

            this.$Input = null;

            this.addEvents({
                onImport: this.$onImport
            });
        },

        /**
         * Event: onImport
         */
        $onImport: function () {
            var self = this;

            this.$Input = this.getElm();
            this.$Input.setStyle('display', 'none');

            var selectedColumnNames = this.$Input.value.split(',');

            this.$CheckBoxesElm = new Element('div', {
                'class': 'quiqqer-locations-settings-listcolumns'
            }).inject(this.$Input, 'after');

            Locations.getAttributeList().then(function (attributes) {
                for (var i = 0, len = attributes.length; i < len; i++) {
                    var attribute = attributes[i];

                    var Label = new Element('label', {
                        'class': 'quiqqer-locations-settings-listcolumns-label',
                        html   : attribute.title
                    }).inject(self.$CheckBoxesElm);

                    var Checkbox = new Element('input', {
                        type       : 'checkbox',
                        'class'    : 'quiqqer-locations-settings-listcolumns-checkbox',
                        'data-name': attribute.name,
                        events     : {
                            change: self.$setSettings
                        }
                    }).inject(Label);

                    if (selectedColumnNames.contains(attribute.name)) {
                        Checkbox.checked = true;
                    }
                }
            });
        },

        /**
         * Set settings to setting input
         */
        $setSettings: function () {
            var checkboxes = this.$CheckBoxesElm.getElements(
                '.quiqqer-locations-settings-listcolumns-checkbox'
            );

            var attributeNames = [];

            for (var i = 0, len = checkboxes.length; i < len; i++) {
                if (checkboxes[i].checked) {
                    attributeNames.push(checkboxes[i].getProperty('data-name'));
                }
            }

            this.$Input.value = attributeNames.join(',');
        }
    });
});