/**
 * (Abstract) parent class for all LocatorFilter JavaScript controls
 *
 * @module package/quiqqer/locations/bin/frontend/controls/Locator/AbstractLocatorFilter
 * @author www.pcsg.de (Patrick Müller)
 *
 * @event onChange [FilterData, this] - Fires if the filter changes values
 */
define('package/quiqqer/locations/bin/frontend/controls/Locator/AbstractLocatorFilter', [

    'qui/controls/Control'

], function (QUIControl) {
    "use strict";

    return new Class({
        Extends: QUIControl,
        Type   : 'package/quiqqer/locations/bin/frontend/controls/Locator/AbstractLocatorFilter',

        Binds: [
            '$fireChange',
            'setFilterData',
            'getFilterData'
        ],

        /**
         * Fire onChange event
         */
        $fireChange: function () {
            this.fireEvent('change', [this.getFilterData(), this]);
        },

        /**
         * Set filter data to control
         *
         * IMPORTANT: Must be implemented by Filter control!
         *
         * @param {*} FilterData
         */
        setFilterData: function (FilterData) {
        },

        /**
         * Get filter data
         *
         * IMPORTANT: Must be implemented by Filter control!
         *
         * @return {*}
         */
        getFilterData: function () {
            return false;
        },

        /**
         * Reset filter to initial status
         *
         * IMPORTANT: Must be implemented by Filter control!
         *
         * @return {Promise}
         */
        reset: function() {
        }
    });
});