<?php

namespace QUI\Locations\Attributes\Types;

use QUI;
use QUI\Locations\Attributes\AbstractAttribute;

/**
 * Class Boolean
 *
 * Yes/no attribute
 *
 * @package QUI\Locations
 */
class Boolean extends AbstractAttribute
{
    /**
     * Set a value to the attribute
     *
     * @param mixed $value
     */
    public function setValue($value)
    {
        $this->value = boolval($value);
    }

    /**
     * Validate the value of an attribute
     *
     * @param string|array $value - The input value (from the frontend)
     * @return bool - true, if value is valid; otherwise an exception is thrown
     *
     * @throws QUI\Locations\Exception
     */
    public function validate($value)
    {
        return true;
    }

    /**
     * Get the value that is used in the search cache table for this attribute
     *
     * @param QUI\Locale $Locale (optional) - Value depending on specific Locale (language);
     * if omitted QUI\Locations\Handler\Locations::getLocale is used
     * @return string - Database value
     */
    public function getSearchCacheValue($Locale = null)
    {
        return $this->value ? 1 : 0;
    }

    /**
     * Get the value that is used in the search cache table for this attribute
     *
     * @param QUI\Locale $Locale (optional) - Value depending on specific Locale (language);
     * if omitted QUI\Locations\Handler\Locations::getLocale is used
     * @return string - Database value
     */
    public function getViewValue($Locale = null)
    {
        return $this->value ? true : false;
    }
}
