<?php

namespace QUI\Locations\Attributes\Types;

use QUI;
use QUI\Locations\Attributes\AbstractAttribute;
use QUI\Locations\Handler\Locations as LocationsHandler;

/**
 * Class ContactInformation
 *
 * Attribute that contains different pieces of contact informatil (phone, email, website etc.)
 *
 * @package QUI\Locations
 */
class ContactInformation extends AbstractAttribute
{
    /**
     * Validate the value of an attribute
     *
     * @param string|array $value - The input value (from the frontend)
     * @return bool - true, if value is valid; otherwise an exception is thrown
     *
     * @throws QUI\Locations\Exception
     */
    public function validate($value)
    {
        if (empty($value)) {
            return true;
        }

        if (is_string($value)) {
            $value = json_decode($value, true);

            if (json_last_error() !== JSON_ERROR_NONE) {
                throw new QUI\Locations\Exception(array(
                    'quiqqer/locations',
                    'exception.attributes.types.datatype.json.error'
                ));
            }
        }

        if (!is_array($value)) {
            throw new QUI\Locations\Exception(array(
                'quiqqer/locations',
                'exception.attributes.types.datatype.array.expected'
            ));
        }

        foreach ($value as $info) {
            switch ($info['type']) {
                case 'phone':
                case 'mobile':
                case 'fax':
                case 'email':
                case 'website':
                    break;

                default:
                    throw new QUI\Locations\Exception(array(
                        'quiqqer/locations',
                        'exception.attributes.types.datatype.wrong'
                    ));
            }

            if (!is_string($info['value'])) {
                throw new QUI\Locations\Exception(array(
                    'quiqqer/locations',
                    'exception.attributes.types.datatype.wrong'
                ));
            }
        }

        return true;
    }

    /**
     * Get the value that is used in the search cache table for this attribute
     *
     * @param QUI\Locale $Locale (optional) - Value depending on specific Locale (language);
     * if omitted QUI\Locations\Handler\Locations::getLocale is used
     * @return string - Database value
     */
    public function getSearchCacheValue($Locale = null)
    {
        if (empty($this->value)) {
            return null;
        }

        $values      = array();
        $contactInfo = json_decode($this->value, true);

        foreach ($contactInfo as $info) {
            $values[] = $info['value'];
        }

        if (empty($values)) {
            return '';
        }

        return ',' . implode(',', $values) . ',';
    }

    /**
     * Get the value that is used in the search cache table for this attribute
     *
     * @param QUI\Locale $Locale (optional) - Value depending on specific Locale (language);
     * if omitted QUI\Locations\Handler\Locations::getLocale is used
     * @return string - Database value
     */
    public function getViewValue($Locale = null)
    {
        return json_decode($this->value, true);
    }
}
