<?php

namespace QUI\Locations\Attributes\Types;

use QUI;
use QUI\Locations\Attributes\AbstractAttribute;
use QUI\Projects\Media\Utils as MediaUtils;

/**
 * Class Image
 *
 * Attribute that contains a quiqqer media image url
 *
 * @package QUI\Locations
 */
class Image extends AbstractAttribute
{
    /**
     * Special set value for this attribute
     *
     * @param mixed $value
     */
    public function setValue($value)
    {
        try {
            parent::setValue($value);
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            $this->value = null;
            return;
        }

        $this->value = str_replace('&amp;', '&', $this->value);
    }

    /**
     * Validate the value of an attribute
     *
     * @param string|array $value - The input value (from the frontend)
     * @return bool - true, if value is valid; otherwise an exception is thrown
     *
     * @throws QUI\Locations\Exception
     */
    public function validate($value)
    {
        if (empty($value)) {
            return true;
        }

        if (!is_string($value)) {
            throw new QUI\Locations\Exception(array(
                'quiqqer/locations',
                'exception.attributes.types.datatype.string.expected'
            ));
        }

        $value = str_replace('&amp;', '&', $value);

        try {
            $Item = MediaUtils::getMediaItemByUrl($value);

            if (!MediaUtils::isImage($Item)) {
                throw new QUI\Locations\Exception();
            }
        } catch (\Exception $Exception) {
            throw new QUI\Locations\Exception(array(
                'quiqqer/locations',
                'exception.attributes.types.image.invalid.url'
            ));
        }

        return true;
    }

    /**
     * Get the value that is used in the search cache table for this attribute
     *
     * @param QUI\Locale $Locale (optional) - Value depending on specific Locale (language);
     * if omitted QUI\Locations\Handler\Locations::getLocale is used
     * @return string - Database value
     */
    public function getSearchCacheValue($Locale = null)
    {
        // has no search cache value
        return null;
    }

    /**
     * Get the value that is used in the search cache table for this attribute
     *
     * @param QUI\Locale $Locale (optional) - Value depending on specific Locale (language);
     * if omitted QUI\Locations\Handler\Locations::getLocale is used
     * @return string - Database value
     */
    public function getViewValue($Locale = null)
    {
        if (empty($this->value)) {
            return '';
        }

        try {
            /** @var QUI\Projects\Media\Image $Item */
            $Item = MediaUtils::getMediaItemByUrl($this->value);
            return $Item->getUrl(true);
        } catch (\Exception $Exception) {
            QUI\System\Log::addError(
                self::class . ' :: getViewValue -> Could not load image url -> ' . $Exception->getMessage()
            );

            return '';
        }
    }
}
