<?php

namespace QUI\Locations\Controls;

use QUI;
use QUI\LocationCategories\Handler as LocationCategories;

/**
 * Class MapLocationDetails
 *
 * Shows details of a Location in a map info window
 */
class MapLocationDetails extends QUI\Control
{
    /**
     * constructor
     *
     * @param array $attributes
     */
    public function __construct($attributes = [])
    {
        $this->setAttributes([
            'Location' => false,
            'Site'     => false
        ]);

        parent::__construct($attributes);
    }

    /**
     * (non-PHPdoc)
     *
     * @throws QUI\Exception
     * @see \QUI\Control::create()
     *
     */
    public function getBody()
    {
        $Engine = QUI::getTemplateManager()->getEngine();
        $css    = \file_get_contents(\dirname(__FILE__) . '/MapLocationDetails.css');

        /** @var QUI\Locations\Location\Location $Location */
        $Location = $this->getAttribute('Location');

        if (empty($Location) || !$Location->isActive()) {
            return '<p>' . QUI::getLocale()->get('quiqqer/locations', 'Locator.MapLocationDetails.inactive') . '</p>';
        }

        $LocationView = $Location->getView();
        $location     = $LocationView->toArray();

        // Determine what is shown in the info layer
        $Site        = $this->getSite();
        $showDetails = [
            'showImages'       => $Site->getAttribute('quiqqer.locations.locator.map.info_layer.show_images'),
            'showAddress'      => $Site->getAttribute('quiqqer.locations.locator.map.info_layer.show_address'),
            'showContact'      => $Site->getAttribute('quiqqer.locations.locator.map.info_layer.show_contact'),
            'showOpeningHours' => $Site->getAttribute('quiqqer.locations.locator.map.info_layer.show_openinghours')
        ];

        // Parse address
        $addressLines = $LocationView->getAddressLines();
        $Engine->assign('addressLines', $addressLines);

        if (empty($addressLines)) {
            $showDetails['showAddress'] = false;
        }

        // Contact
        if (empty($location['contactInformation'])) {
            $showDetails['showContact'] = false;
        }

        // Determine which columns are shown
        if (!empty($showDetails['showOpeningHours']) && !empty($location['openingHours']['openingLines'])) {
            $leftColumnCssClass  = 'quiqqer-locations-locator-maplocationdetails-column-55';
            $rightColumnCssClass = 'quiqqer-locations-locator-maplocationdetails-column-45';
            $maxWidth            = 'quiqqer-locations-locator-maplocationdetails__400';
        } else {
            $leftColumnCssClass  = 'quiqqer-locations-locator-maplocationdetails-column-100';
            $rightColumnCssClass = 'quiqqer-locations-locator-maplocationdetails-column-0';
            $maxWidth            = 'quiqqer-locations-locator-maplocationdetails__200';
        }

        $Engine->assign([
            'leftColumnCssClass'  => $leftColumnCssClass,
            'rightColumnCssClass' => $rightColumnCssClass,
            'maxWidth'            => $maxWidth
        ]);

        if (!empty($location['mainImage'])) {
            /** @var QUI\Projects\Media\Image $MainImage */
            $locationImage = $Location->getLocationAttribute('mainImage')->getValue();
            $LocationImage = QUI\Projects\Media\Utils::getImageByUrl($locationImage);

            $location['mainImage'] = $LocationImage->getSizeCacheUrl(200, 200);
        } else {
            $showDetails['showImages'] = false;
        }

        // Category
        $categoryTitle = false;

        if (QUI::getPackageManager()->isInstalled('quiqqer/locationcategories')) {
            try {
                $categoryTitle = LocationCategories::getCategoryByLocation($Location)->getTitle();
            } catch (\Exception $Exception) {
                QUI\System\Log::writeException($Exception);
            }
        }

        $Engine->assign($showDetails);
        $Engine->assign('categoryTitle', $categoryTitle);
        $Engine->assign('locationUrl', $LocationView->getUrl());
        $Engine->assign('location', $location);
        $Engine->assign('css', $css);

        return $Engine->fetch(\dirname(__FILE__) . '/MapLocationDetails.html');
    }

    /**
     * Return the current site
     *
     * @return mixed|QUI\Projects\Site
     *
     * @throws QUI\Exception
     */
    protected function getSite()
    {
        if ($this->getAttribute('Site')) {
            return $this->getAttribute('Site');
        }

        return QUI::getRewrite()->getSite();
    }
}
