<?php

namespace QUI\Locations;

use QUI;
use QUI\Locations\Handler\LocationAPIs as APIHandler;
use Cron\CronExpression;

/**
 * Class Crons
 *
 * quiqqer/locations cronjobs
 */
class Crons
{
    /**
     * Execute all location APIs that are eligible for execution
     *
     * @return void
     */
    public static function executeLocationApis()
    {
        $list = APIHandler::getList();

        foreach ($list as $entry) {
            $execTime = json_decode($entry['execTime'], true);

            if (!$execTime['enabled']) {
                continue;
            }

            $Cron = CronExpression::factory(
                implode(" ", $execTime['time'])
            );

            $lastExec     = $entry['lastExec'];
            $nowTimestamp = time();

            if (is_null($lastExec)) {
                $lastExec = '1970-01-01 00:00:00';
            } else {
                $lastExec = date('Y-m-d H:i:s', $lastExec);
            }

            $next = $Cron->getNextRunDate($lastExec)->getTimestamp();

            // no execute
            if ($next > $nowTimestamp) {
                continue;
            }

            APIHandler::execute($entry['id']);
        }
    }
}
